#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# opensecurityd
# 
# the opensecurityd as RESTful server
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright (C) 2013 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import os
import os.path
import subprocess
import sys
import web

from vmmanager import VMManager

# local
from environment import Environment


# ------------------------------------------------------------
# const

__version__ = "0.1"


"""All the URLs we know mapping to class handler"""
opensecurity_urls = (
    '/device_change',           'os_device_change',     # http://localhost:8080/device_change   GET
    '/sdvm_started',            'os_sdvm_started',      # http://localhost:8080/sdvm_started    GET
    '/sdvms',                   'os_sdvms',             # http://localhost:8080/sdvms           GET, PUT
    '/sdvms/(.*)',              'os_sdvm',              # http://localhost:8080/sdvms           GET, DELETE
    '/vms',                     'os_vms',               # http://localhost:8080/vms             GET
    '/vms/(.*)',                'os_vm',                # http://localhost:8080/vms/[VMNAME]    GET
    '/',                        'os_root'               # http://localhost:8080/                GET
)


# ------------------------------------------------------------
# vars

# Global VMManager instance
gvm_mgr = VMManager.getInstance()

# ------------------------------------------------------------
# code


class os_device_change:
    """OpenSecurity '/device_change' handler"""
    
    def GET(self):
        gvm_mgr.handleDeviceChange()
        return "os_device_change"

class os_sdvm_started:
    """OpenSecurity '/sdvm_started' handler"""
    
    def GET(self):
        # self.request get address
        print "os_sdvm_started"
        return "os_sdvm_started"

class os_sdvm:
    """OpenSecurity '/sdvms/[VM]' handler"""
    
    def GET(self, name):
        return gvm_mgr.getVMInfo(name)
            

    def DELETE(self, name):
        return gvm_mgr.removeVM(name)
            

class os_sdvms:
    """OpenSecurity '/sdvms' handler"""
    
    def GET(self):
        """get the list of SDVMs"""
        return gvm_mgr.listSDVM() 
            
    def PUT(self):
        """create a new SDVM"""

        # pick the vm-name
        args = web.input()
        if not "name" in args:
            raise web.badrequest()
            
        return gvm_mgr.createVM(args.name)
            
class os_vm:
    """OpenSecurity '/vms/[VM]' handler"""
    
    def GET(self, name):
        return gvm_mgr.getVMInfo(name)
            

class os_vms:
    """OpenSecurity '/vms' handler"""
    
    def GET(self):
        return gvm_mgr.listVM() 
            

class os_root:
    """OpenSecurity '/' handler"""
    
    def GET(self):
        res = "'os_server': { "
        res += "'version': '" + __version__ + "', "
        res += "'machine_folder': '" + gvm_mgr.getDefaultMachineFolder() + "' "
        res += "}"
        return res


# start
if __name__ == "__main__":
    server = web.application(opensecurity_urls, globals())
    server.run()

